﻿using RootPro.RootProCAD;
using System;
using System.Collections.Generic;
using System.IO;

namespace RCAddInRegisterFiltersCS
{
    partial class AppAddIn
    {
        // FilterSettingsData クラスの保存XMLファイル名
        public string RC_FilterSettingsDataXmlFileName = "RCAddInRegisterFiltersSettings.xml";



        /// <summary>
        /// 条件選択データ
        /// </summary>
        [Serializable]
        public class FilterData
        {
            // フィルタ名
            public string filterName;

            // 図形フィルタ
            public ShapeTypeFilters shapeTypeFilters;

            // 色フィルタ
            public List<int> colorFilterNumbers;

            // 線種フィルタ
            public List<int> lineTypeFilterNumbers;

            // 線幅フィルタ
            public List<int> lineWidthFilterNumbers;

            /// <summary>
            /// コンストラクタ―
            /// </summary>
            public FilterData()
            {
                filterName = "";

                // 図形フィルタ
                shapeTypeFilters = 0;

                // 色フィルタ
                colorFilterNumbers = new List<int>();

                // 線種フィルタ
                lineTypeFilterNumbers = new List<int>();
              
                // 線幅フィルタ
                lineWidthFilterNumbers = new List<int>();
            }

            /// <summary>
            /// クローン
            /// </summary>
            public FilterData Clone()
            {
                return (FilterData)MemberwiseClone();
            }

            /// <summary>
            /// 色フィルタを追加する
            /// </summary>
            public void AddColorFilter(int index)
            {
                if(!colorFilterNumbers.Contains(index))
                {
                    colorFilterNumbers.Add(index);
                    colorFilterNumbers.Sort();
                }
            }

            /// <summary>
            /// 線種フィルタを追加する
            /// </summary>
            public void AddLineTypeFilter(int index)
            {
                if(!lineTypeFilterNumbers.Contains(index))
                {
                    lineTypeFilterNumbers.Add(index);
                    lineTypeFilterNumbers.Sort();
                }
            }

            /// <summary>
            /// 線幅フィルタを追加する
            /// </summary>
            public void AddLineWidthFilter(int index)
            {
                if(!lineWidthFilterNumbers.Contains(index))
                {
                    lineWidthFilterNumbers.Add(index);
                    lineWidthFilterNumbers.Sort();
                }
            }
        }

        /// <summary>
        /// 条件選択のデータクラス
        /// </summary>
        public class FilterSettingsData
        {
            // 条件選択データのリスト
            public List<FilterData> filterList;

            /// <summary>
            /// コンストラクタ―
            /// </summary>
            public FilterSettingsData()
            {
                filterList = new List<FilterData>();
            }

            /// <summary>
            /// 条件選択データを追加する
            /// </summary>
            /// <param name="data"></param>
            public void AddFilterData(FilterData data)
            {
                filterList.Add(data);
            }

            /// <summary>
            /// フィルタ名でFilterData を取得
            /// </summary>
            /// <param name="name"></param>
            public FilterData GetFilterData(string name)
            {
                // 名前が一致するフィルタを返す
                foreach(var filter in filterList)
                {
                    if(filter.filterName.Equals(name))
                        return filter;
                }
                return null;
            }

            /// <summary>
            /// フィルタのリストを取得
            /// </summary>
            public List<FilterData> GetFilterList()
            {
                return filterList;
            }

            /// <summary>
            /// フィルタのリストを設定
            /// </summary>
            /// <param name="list"></param>
            public void SetFilterList(List<FilterData> list)
            {
                filterList = list;
            }

            /// <summary>
            /// フィルタ名でFilterData を削除
            /// </summary>
            /// <param name="name"></param>
            public bool DeleteFilterData(string name)
            {
                // 名前が一致するフィルタを返す
                foreach(var filter in filterList)
                {
                    if(filter.filterName.Equals(name))
                    {
                        // 一致するものをリストから削除
                        filterList.Remove(filter);
                        return true;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 設定値 FilterSettingsData をファイルに保存
        /// </summary>
        /// <param name="data"></param>
        public void SaveFilterSettingsData(FilterSettingsData data)
        {
            // 保存ファイルパス
            string addinDir = Path.GetDirectoryName(AddInFullPath);
            string filePathName = addinDir + "\\" + RC_FilterSettingsDataXmlFileName;

            System.IO.StreamWriter sw = null;

            try
            {
                // 書き込むファイルを開く（UTF-8 BOM無し）
                sw = new System.IO.StreamWriter(filePathName, false, new System.Text.UTF8Encoding(false));

                // XmlSerializerオブジェクトを作成
                // CustomSettings
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(FilterSettingsData));

                //シリアル化し、XMLファイルに保存する
                serializer.Serialize(sw, data);

                //ファイルを閉じる
                sw.Close();
            }
            catch(System.Exception)
            {

            }
            finally
            {
                if(sw != null)
                {
                    sw.Close();
                }
            }
        }

        /// <summary>
        /// 条件選択の設定をファイルから読み込む
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public bool LoadFilterSettingsData(out FilterSettingsData data)
        {
            data = null;

            // 保存ファイルパス
            string addinDir = Path.GetDirectoryName(AddInFullPath);
            string filePathName = addinDir + "\\" + RC_FilterSettingsDataXmlFileName;

            //読み込むファイルを開く
            StreamReader sr = null;

            try
            {
                sr = new StreamReader(filePathName, new System.Text.UTF8Encoding(false));

                // XmlSerializerオブジェクトを作成
                // FilterSettingsData
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(FilterSettingsData));

                // XMLファイルから読み込み、逆シリアル化する
                data = (FilterSettingsData)serializer.Deserialize(sr);

                //ファイルを閉じる
                sr.Close();

                return true;
            }
            catch(System.Exception)
            {

            }
            finally
            {
                if(sr != null)
                {
                    sr.Close();
                }
            }

            return false;
        }
    }
}